@@ -1136,7 +1136,7 @@ struct runGapOBJ: public RcppParallel::Worker
     vec<indtype> NnodesKps(maxCore * 2, 0);
     nodes = &NnodesKps[0];
     kps = &NnodesKps[maxCore];
-    parallelFor(0, maxCore, *this);
+      parallelFor(0, dT->NofCore, *this);
     totalNnodes = std::accumulate(nodes, nodes + maxCore, totalNnodes);
     totalNkps = std::accumulate(kps, kps + maxCore, totalNkps);
   }
@@ -1303,6 +1303,8 @@ List auxGAPbbDpMulthreadNodes(IntegerMatrix cost, NumericMatrix profitOrLoss, In
 
 
 
+
+
 
 
 
